@@ -5,6 +5,8 @@ from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
 from bergen.actors.utils import *
 import janus
 import asyncio
+import threading
+
 
 class FunctionalActor(Actor):
     pass
@@ -19,7 +21,7 @@ class FunctionalFuncActor(FunctionalActor):
         raise NotImplementedError("Please provide a func or overwrite the assign method!")
 
     async def _assign(self, assign_handler: AssignHandler, args, kwargs):
-    
+        bounce_context.set(assign_handler.message.meta.context)
         assign_handler_context.set(assign_handler)
         provide_handler_context.set(self.provide_handler)
         #
@@ -41,7 +43,7 @@ class FunctionalGenActor(FunctionalActor):
         raise NotImplementedError("This needs to be overwritten in order to work")
 
     async def _assign(self, assign_handler: AssignHandler, args, kwargs):
-
+        bounce_context.set(assign_handler.message.meta.context)
         assign_handler_context.set(assign_handler)
         provide_handler_context.set(self.provide_handler)
 
@@ -126,11 +128,19 @@ class FunctionalThreadedGenActor(FunctionalActor):
         raise NotImplementedError("")  
 
 
-    def _assign_threaded(self, args, kwargs, queue, context):
+    def _assign_threaded(self, args, kwargs, queue, event, context):
         queue_context.set(queue)
         bounce_context.set(context)
         try:
+            if event.is_set(): 
+                queue.put(("cancelled","Happy doneness")) 
+                return
+
             for result in self.assign(*args, **kwargs):
+                if event.is_set():
+                    queue.put(("cancelled","Happy doneness"))
+                    return
+                    
                 lastresult = shrinkOutputsSync(self.template.node, result) if self.shrinkOutputs else result
                 queue.put(("yield", lastresult))
                 queue.join()
@@ -138,12 +148,13 @@ class FunctionalThreadedGenActor(FunctionalActor):
             console.print_exception()
             queue.put(("exception", e))
 
-        queue.put(("done"))
+        queue.put(("done","Happy doneness"))
         queue_context.set(None)
         bounce_context.set(None)
 
 
     async def iterate_queue(self, async_q, assign_handler: AssignHandler, provide_handler):
+
         while True:
             val = await async_q.get()
             action = val[0]
@@ -161,18 +172,36 @@ class FunctionalThreadedGenActor(FunctionalActor):
                 await assign_handler.pass_done()
                 async_q.task_done()
                 break
+
         
 
-
-
     async def _assign(self, assign_handler: AssignHandler, args, kwargs):
         queue = janus.Queue()
         print(assign_handler.message.meta.context)
+
+        event = threading.Event()
+
         try:
-            fut = self.loop.run_in_executor(self.threadpool, self._assign_threaded, args, kwargs, queue.sync_q, assign_handler.message.meta.context)
-            await self.iterate_queue(queue.async_q, assign_handler, self.provide_handler)
-            await fut
-            queue.close()
-            await queue.wait_closed()
+            threadedfut = self.loop.run_in_executor(self.threadpool, self._assign_threaded, args, kwargs, queue.sync_q, event, assign_handler.message.meta.context)
+            queuefut =  self.iterate_queue(queue.async_q, assign_handler, self.provide_handler)
+
+            try:
+                await asyncio.gather(threadedfut, queuefut)
+                queue.close()
+                await queue.wait_closed()
+            except asyncio.CancelledError as e:
+                console.log("Received Cancellation for task")
+
+                if not threadedfut.done():
+                    print("Sending request to Queue To Cancell")
+                    event.set()
+                    threadedfut.cancel()
+
+                try:
+                    await threadedfut
+                except asyncio.CancelledError as e:
+                    print("Sucessfully Cancelled Thread")
+                    raise e
+
         except Exception as e:
             await assign_handler.pass_exception(e)
\ No newline at end of file
