@@ -1,7 +1,7 @@
 from .assign import AssignMessage
 from .assign_cancelled import AssignCancelledMessage
 from .assign_critical import AssignCriticalMessage
-from .assign_progress import AssignProgressMessage
+from .assign_log import AssignLogMessage
 from .assign_return import AssignReturnMessage
 from .bounced_assign import BouncedAssignMessage
 from .assign_yield import AssignYieldsMessage
